import schema_crawler_oracle
import unittest

class TestClass(unittest.TestCase):

    def test_oracle_parse_url(self):
        self.maxDiff = None
        self.assertEqual(schema_crawler_oracle.DbURL('user', 'pass', 'host', 'port', 'svc'), schema_crawler_oracle._oracle_parse_url("user/pass@//host:port/svc"))
        self.assertEqual(schema_crawler_oracle.DbURL('user', 'pass', 'host', 'port', 'svc'), schema_crawler_oracle._oracle_parse_url("user/pass@host:port/svc"))
        self.assertEqual(schema_crawler_oracle.DbURL('user', 'pass', 'host', '1521', 'svc'), schema_crawler_oracle._oracle_parse_url("user/pass@//host/svc"))
        self.assertEqual(schema_crawler_oracle.DbURL('user', 'pass', 'host', '1521', 'svc'), schema_crawler_oracle._oracle_parse_url("user/pass@host/svc"))
        self.assertRaises(Exception, schema_crawler_oracle._oracle_parse_url, "pass@//host/svc")
        self.assertRaises(Exception, schema_crawler_oracle._oracle_parse_url, "user/pass@//svc")
        self.assertRaises(Exception, schema_crawler_oracle._oracle_parse_url, "user/pass@svc")

    def test_oracle_split_create_index_user_bug1(self):
        self.maxDiff = None
        output = schema_crawler_oracle._oracle_split_manually("""
  CREATE TABLE "IDA_MAIN"."ACCOUNT_CLASS_DIM" 
   (    "ACCOUNT_CLASS_DIM_ID" NUMBER NOT NULL ENABLE, 
        "EFFECTIVE_DATE" TIMESTAMP (6) NOT NULL ENABLE, 
        "LOAD_ID" NUMBER NOT NULL ENABLE, 
        "ACTIVE_IND" VARCHAR2(10 CHAR) NOT NULL ENABLE, 
        "ACCOUNT_CLASS_CODE" VARCHAR2(10 CHAR) NOT NULL ENABLE, 
        "ACCOUNT_CLASS_DESC" VARCHAR2(80 CHAR) NOT NULL ENABLE
   ) SEGMENT CREATION IMMEDIATE 
  PCTFREE 10 PCTUSED 40 INITRANS 1 MAXTRANS 255 
 NOCOMPRESS LOGGING
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_DATA"
  CREATE UNIQUE INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_PK" ON "IDA_MAIN"."ACCOUNT_CLASS_DIM" ("ACCOUNT_CLASS_DIM_ID")
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_INDX" 
  CREATE UNIQUE INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_UK" ON "IDA_MAIN"."ACCOUNT_CLASS_DIM" ("ACCOUNT_CLASS_CODE") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_INDX"
ALTER TABLE "IDA_MAIN"."ACCOUNT_CLASS_DIM" ADD CONSTRAINT "ACCOUNT_CLASS_DIM_PK" PRIMARY KEY ("ACCOUNT_CLASS_DIM_ID")
  USING INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_PK"  ENABLE
ALTER TABLE "IDA_MAIN"."ACCOUNT_CLASS_DIM" ADD CONSTRAINT "ACCOUNT_CLASS_DIM_UK" UNIQUE ("ACCOUNT_CLASS_CODE")
  USING INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_UK"  ENABLE
  CREATE UNIQUE INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_PK" ON "IDA_MAIN"."ACCOUNT_CLASS_DIM" ("ACCOUNT_CLASS_DIM_ID")
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_INDX"
  CREATE UNIQUE INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_UK" ON "IDA_MAIN"."ACCOUNT_CLASS_DIM" ("ACCOUNT_CLASS_CODE") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_INDX" 

  ALTER TABLE "IDA_MAIN"."ACCOUNT_CLASS_DIM" ADD CONSTRAINT "ACCOUNT_CLASS_DIM_PK" PRIMARY KEY ("ACCOUNT_CLASS_DIM_ID")
  USING INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_PK"  ENABLE
   """.strip())
        output = [i.strip() for i in output]
        expected = [
        """
CREATE TABLE "IDA_MAIN"."ACCOUNT_CLASS_DIM" 
   (    "ACCOUNT_CLASS_DIM_ID" NUMBER NOT NULL ENABLE, 
        "EFFECTIVE_DATE" TIMESTAMP (6) NOT NULL ENABLE, 
        "LOAD_ID" NUMBER NOT NULL ENABLE, 
        "ACTIVE_IND" VARCHAR2(10 CHAR) NOT NULL ENABLE, 
        "ACCOUNT_CLASS_CODE" VARCHAR2(10 CHAR) NOT NULL ENABLE, 
        "ACCOUNT_CLASS_DESC" VARCHAR2(80 CHAR) NOT NULL ENABLE
   ) SEGMENT CREATION IMMEDIATE 
  PCTFREE 10 PCTUSED 40 INITRANS 1 MAXTRANS 255 
 NOCOMPRESS LOGGING
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_DATA" """.strip(),
"""CREATE UNIQUE INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_PK" ON "IDA_MAIN"."ACCOUNT_CLASS_DIM" ("ACCOUNT_CLASS_DIM_ID")
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_INDX" """.strip(),
"""CREATE UNIQUE INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_UK" ON "IDA_MAIN"."ACCOUNT_CLASS_DIM" ("ACCOUNT_CLASS_CODE") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_INDX" """.strip(),
"""ALTER TABLE "IDA_MAIN"."ACCOUNT_CLASS_DIM" ADD CONSTRAINT "ACCOUNT_CLASS_DIM_PK" PRIMARY KEY ("ACCOUNT_CLASS_DIM_ID")
  USING INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_PK"  ENABLE""".strip(),
"""ALTER TABLE "IDA_MAIN"."ACCOUNT_CLASS_DIM" ADD CONSTRAINT "ACCOUNT_CLASS_DIM_UK" UNIQUE ("ACCOUNT_CLASS_CODE")
  USING INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_UK"  ENABLE""".strip(),
"""CREATE UNIQUE INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_PK" ON "IDA_MAIN"."ACCOUNT_CLASS_DIM" ("ACCOUNT_CLASS_DIM_ID")
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_INDX" """.strip(),
"""CREATE UNIQUE INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_UK" ON "IDA_MAIN"."ACCOUNT_CLASS_DIM" ("ACCOUNT_CLASS_CODE") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "IDA_DIM_INDX" """.strip(),
"""ALTER TABLE "IDA_MAIN"."ACCOUNT_CLASS_DIM" ADD CONSTRAINT "ACCOUNT_CLASS_DIM_PK" PRIMARY KEY ("ACCOUNT_CLASS_DIM_ID")
  USING INDEX "IDA_MAIN"."ACCOUNT_CLASS_DIM_PK"  ENABLE""".strip()
        ]
        self.assertEqual(expected, output)

    def test_oracle_split_create_index(self):
        self.maxDiff = None
        output = schema_crawler_oracle._oracle_split_manually("""
  CREATE UNIQUE INDEX "HR"."EMP_EMAIL_UK" ON "HR"."EMPLOYEES" ("EMAIL") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" 
  CREATE UNIQUE INDEX "HR"."EMP_EMP_ID_PK" ON "HR"."EMPLOYEES" ("EMPLOYEE_ID") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" 
  CREATE INDEX "HR"."EMP_DEPARTMENT_IX" ON "HR"."EMPLOYEES" ("DEPARTMENT_ID") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" 
  CREATE INDEX "HR"."EMP_JOB_IX" ON "HR"."EMPLOYEES" ("JOB_ID") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" 
  CREATE INDEX "HR"."EMP_MANAGER_IX" ON "HR"."EMPLOYEES" ("MANAGER_ID") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" 
  CREATE INDEX "HR"."EMP_NAME_IX" ON "HR"."EMPLOYEES" ("LAST_NAME", "FIRST_NAME") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS"
""")
        expected = [
"""  CREATE UNIQUE INDEX "HR"."EMP_EMAIL_UK" ON "HR"."EMPLOYEES" ("EMAIL") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" """,
"""  CREATE UNIQUE INDEX "HR"."EMP_EMP_ID_PK" ON "HR"."EMPLOYEES" ("EMPLOYEE_ID") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" """,
"""  CREATE INDEX "HR"."EMP_DEPARTMENT_IX" ON "HR"."EMPLOYEES" ("DEPARTMENT_ID") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" """,
"""  CREATE INDEX "HR"."EMP_JOB_IX" ON "HR"."EMPLOYEES" ("JOB_ID") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" """,
"""  CREATE INDEX "HR"."EMP_MANAGER_IX" ON "HR"."EMPLOYEES" ("MANAGER_ID") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" """,
"""  CREATE INDEX "HR"."EMP_NAME_IX" ON "HR"."EMPLOYEES" ("LAST_NAME", "FIRST_NAME") 
  PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS 
  STORAGE(INITIAL 65536 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
  PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1
  BUFFER_POOL DEFAULT FLASH_CACHE DEFAULT CELL_FLASH_CACHE DEFAULT)
  TABLESPACE "USERS" """.rstrip()
        ]
        self.assertEqual(expected, output)

    def test_oracle_split_comments(self):
        self.maxDiff = None
        output = schema_crawler_oracle._oracle_split_manually("""
   COMMENT ON COLUMN "HR"."EMPLOYEES"."EMPLOYEE_ID" IS 'Primary key of employees table.'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."FIRST_NAME" IS 'First name of the employee. A not null column.'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."LAST_NAME" IS 'Last name of the employee. A not null column.'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."EMAIL" IS 'Email id of the employee'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."PHONE_NUMBER" IS 'Phone number of the employee; includes country code and area code'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."HIRE_DATE" IS 'Date when the employee started on this job. A not null column.'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."JOB_ID" IS 'Current job of the employee; foreign key to job_id column of the jobs table. A not null column.'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."SALARY" IS 'Monthly salary of the employee. Must be greater than zero (enforced by constraint emp_salary_min)'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."COMMISSION_PCT" IS 'Commission percentage of the employee; Only employees in sales department elgible for commission percentage'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."MANAGER_ID" IS 'Manager id of the employee; has same domain as manager_id in departments table. Foreign key to employee_id column of employees table. (useful for reflexive joins and CONNECT BY query)'
   COMMENT ON COLUMN "HR"."EMPLOYEES"."DEPARTMENT_ID" IS 'Department id where employee works; foreign key to department_id column of the departments table'
   COMMENT ON TABLE "HR"."EMPLOYEES"  IS 'employees table. Contains 107 rows. References with departments, jobs, job_history tables. Contains a self reference.'
""")
        expected = [
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."EMPLOYEE_ID" IS 'Primary key of employees table.'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."FIRST_NAME" IS 'First name of the employee. A not null column.'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."LAST_NAME" IS 'Last name of the employee. A not null column.'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."EMAIL" IS 'Email id of the employee'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."PHONE_NUMBER" IS 'Phone number of the employee; includes country code and area code'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."HIRE_DATE" IS 'Date when the employee started on this job. A not null column.'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."JOB_ID" IS 'Current job of the employee; foreign key to job_id column of the jobs table. A not null column.'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."SALARY" IS 'Monthly salary of the employee. Must be greater than zero (enforced by constraint emp_salary_min)'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."COMMISSION_PCT" IS 'Commission percentage of the employee; Only employees in sales department elgible for commission percentage'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."MANAGER_ID" IS 'Manager id of the employee; has same domain as manager_id in departments table. Foreign key to employee_id column of employees table. (useful for reflexive joins and CONNECT BY query)'""",
"""   COMMENT ON COLUMN "HR"."EMPLOYEES"."DEPARTMENT_ID" IS 'Department id where employee works; foreign key to department_id column of the departments table'""",
"""   COMMENT ON TABLE "HR"."EMPLOYEES"  IS 'employees table. Contains 107 rows. References with departments, jobs, job_history tables. Contains a self reference.'"""
        ]
        self.assertEqual(expected, output)

if __name__ == '__main__':
    unittest.main()
